@@ -22,6 +22,11 @@ struct pt_or_px {
     float pt;
 };
 
+struct font_size_adjustment {
+    struct pt_or_px pt_or_px;
+    float percent;
+};
+
 enum cursor_style { CURSOR_BLOCK, CURSOR_UNDERLINE, CURSOR_BEAM };
 
 enum conf_size_type {CONF_SIZE_PX, CONF_SIZE_CELLS};
@@ -69,11 +74,6 @@ enum key_binding_type {
     MOUSE_BINDING,
 };
 
-struct config_key_binding_text {
-    char *text;
-    bool master_copy;
-};
-
 struct config_key_binding {
     int action;  /* One of the varios bind_action_* enums from wayland.h */
     struct config_key_modifiers modifiers;
@@ -139,6 +139,7 @@ struct config {
 
     enum {DPI_AWARE_AUTO, DPI_AWARE_YES, DPI_AWARE_NO} dpi_aware;
     struct config_font_list fonts[4];
+    struct font_size_adjustment font_size_adjustment;
 
     /* Custom font metrics (-1 = use real font metrics) */
     struct pt_or_px line_height;
@@ -150,6 +151,7 @@ struct config {
 
     bool use_custom_underline_offset;
     struct pt_or_px underline_offset;
+    struct pt_or_px underline_thickness;
 
     bool box_drawings_uses_font_glyphs;
     bool can_shape_grapheme;
@@ -318,6 +320,8 @@ struct config {
 
     env_var_list_t env_vars;
 
+    char *utempter_path;
+
     struct {
         enum fcft_scaling_filter fcft_filter;
         bool overflowing_glyphs;
