@@ -96,7 +96,7 @@ enum damage_type {DAMAGE_SCROLL, DAMAGE_SCROLL_REVERSE,
 struct damage {
     enum damage_type type;
     struct scroll_region region;
-    int lines;
+    uint16_t lines;
 };
 
 struct row_uri_range {
@@ -598,6 +598,15 @@ struct terminal {
         struct timespec input_time;
     } render;
 
+    struct {
+        struct grid *grid;    /* Original ‘normal’ grid, before resize started */
+        int old_screen_rows;  /* term->rows before resize started */
+        int old_cols;         /* term->cols before resize started */
+        int old_hide_cursor;  /* term->hide_cursor before resize started */
+        int new_rows;         /* New number of scrollback rows */
+        struct range selection_coords;
+    } interactive_resizing;
+
     struct {
         enum {
             SIXEL_DECSIXEL,  /* DECSIXEL body part ", $, -, ? ... ~ */
@@ -805,6 +814,9 @@ void term_collect_urls(struct terminal *term);
 void term_osc8_open(struct terminal *term, uint64_t id, const char *uri);
 void term_osc8_close(struct terminal *term);
 
+bool term_ptmx_pause(struct terminal *term);
+bool term_ptmx_resume(struct terminal *term);
+
 static inline void term_reset_grapheme_state(struct terminal *term)
 {
 #if defined(FOOT_GRAPHEME_CLUSTERING)
